// Title:   FeatureList.cpp
// Created: Thu Sep 25 15:07:33 2003
// Authors: Richard Keiser, Mark Pauly
//
// Copyright (c) 2001, 2002, 2003, Computer Graphics Lab, ETH Zurich
//
// This file is part of the Pointshop3D system.
// See http://www.pointshop3d.com/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact info@pointshop3d.com if any conditions of this
// licensing are not clear to you.
//

#include "FeatureList.h"


FeatureList::FeatureList () {

	firstObj = 0;
	lastObj = 0;
	currentObj = 0;
	nObjs = 0;
	bAutoDelete = false;
}


FeatureList::~FeatureList () {

	clear();
}


unsigned int FeatureList::count () const {

	return nObjs;
}


bool FeatureList::isEmpty () const {

	return firstObj ? true : false;
}


void FeatureList::append (FeaturePoint *obj) {

	nObjs++;
	
	Element* el = new Element;
	el->obj = obj;
	el->isDeleted = false;

	if (lastObj == 0) {
		firstObj = el;
		firstObj->prev = NULL;
		firstObj->next = NULL;
	} 
	else {
		lastObj->next = el;
		el->prev = lastObj;
		el->next = 0;
	}
	lastObj = el;

	currentObj = el;
}


void FeatureList::prepend (FeaturePoint *obj) {

	nObjs++;

	Element *el = new Element;
	el->obj = obj;
	el->isDeleted = false;

	if (firstObj == 0) {
		el->next = 0;
		el->prev = 0;

		lastObj = el;
	} 
	else {
		firstObj->prev = el;
		el->next = firstObj;
		el->prev = 0;
	}
	firstObj = el;

	currentObj = el;
}


void FeatureList::insert (FeaturePoint *obj) {
	
	if (currentObj == 0) {
		append(obj);
		return;
	}
	
	nObjs++;

	Element *el = new Element;
	el->obj = obj;
	el->isDeleted = false;
	el->next = currentObj->next;
	el->prev = currentObj;

	el->next->prev = el;
	currentObj->next = el;
	currentObj = el;
}

bool FeatureList::setAsDeleted () {

	if (currentObj) {
		currentObj->isDeleted = true;
		return true;
	}
	return false;
}

bool FeatureList::remove () {

	if (currentObj) {
		Element *el = currentObj;

		nObjs--;

		if(currentObj == firstObj) {
			firstObj = currentObj->next;
			firstObj->prev = 0;
			currentObj = currentObj->next;

			if(currentObj == lastObj) {
				lastObj = 0;
			}
		} 
		else {
			currentObj->prev->next = currentObj->next;

			if (currentObj == lastObj) {
				lastObj = currentObj->prev;
				lastObj->next = 0;
				currentObj = lastObj;
			}
			else {
				currentObj->next->prev = currentObj->prev;
				currentObj = currentObj->next;
			}
		}

		if (bAutoDelete) {
			delete el->obj;
		}

		delete el;

		return true;
	}

	return false;
}


void FeatureList::clear () {

	Element *obj,
			*tmp;

	obj = firstObj;
	while(obj != 0) {
		tmp = obj;
		obj = obj->next;
		
		if (bAutoDelete) {
			delete tmp->obj;
		}
		delete tmp;
	}

	firstObj = 0;
	lastObj = 0;
	currentObj = 0;
	nObjs = 0;
}


FeaturePoint * FeatureList::current () {

	while (currentObj && currentObj->isDeleted) {
		currentObj = currentObj->next;
	}
	if (currentObj) {
		return currentObj->obj;
	}
	return NULL;
}


FeaturePoint* FeatureList::getNext () const {

	if (currentObj) {

		Element *tmp = currentObj->next;
		while (tmp && tmp->isDeleted) {
			tmp = tmp->next;
		}

		if (tmp) {
			return tmp->obj;
		}
	}

	return NULL;
}


FeaturePoint* FeatureList::getPrev () const {

	if (currentObj) {

		Element *tmp = currentObj->prev;
		while (tmp && tmp->isDeleted) {
			tmp = tmp->prev;
		}

		if (tmp) {
			return tmp->obj;
		}
	}

	return NULL;
}

FeaturePoint* FeatureList::getFirst () const {

	Element *tmp = firstObj;
	while (tmp && tmp->isDeleted) {
		tmp = tmp->next;
	}
	if (tmp) {
		return tmp->obj;
	}
	else {
		return NULL;
	}
}


FeaturePoint* FeatureList::getLast () const {

	Element *tmp = lastObj;
	while (tmp && tmp->isDeleted) {
		tmp = tmp->prev;
	}
	if (tmp) {
		return tmp->obj;
	}
	else {
		return NULL;
	}
}



FeaturePoint* FeatureList::first () {

	currentObj = firstObj;
	while (currentObj && currentObj->isDeleted) {
		currentObj = currentObj->next;
	}
	if (currentObj) {
		return currentObj->obj;
	}
	else {
		return NULL;
	}
}


FeaturePoint* FeatureList::last () {

	currentObj = lastObj;
	while (currentObj && currentObj->isDeleted) {
		currentObj = currentObj->prev;
	}
	if (currentObj) {
		return currentObj->obj;
	}
	else {
		return NULL;
	}
}


FeaturePoint* FeatureList::next () {

	while (currentObj->next && currentObj->next->isDeleted) {
		currentObj = currentObj->next;
	}

	if (currentObj->next) {
		currentObj = currentObj->next;
		return currentObj->obj;
	} 
		
	return 0;
}


FeaturePoint * FeatureList::prev () {

	
	while (currentObj->prev && currentObj->prev->isDeleted) {
		currentObj = currentObj->prev;
	}

	if (currentObj->prev) {
		currentObj = currentObj->prev;
		return currentObj->obj;
	} 
		
	return 0;
}


FeaturePoint* FeatureList::getNext (bool &isDeleted) const {

	if (currentObj && currentObj->next) {
		isDeleted = currentObj->next->isDeleted;
		return currentObj->next->obj;
	}
	else {
		isDeleted = false;
		return NULL;
	}
}

FeaturePoint* FeatureList::getPrev (bool &isDeleted) const {

	if (currentObj && currentObj->prev) {
		isDeleted = currentObj->prev->isDeleted;
		return currentObj->prev->obj;
	}
	else {
		isDeleted = false;
		return NULL;
	}
}


FeaturePoint* FeatureList::first (bool &isDeleted) {

	currentObj = firstObj;
	if (currentObj) {
		isDeleted = currentObj->isDeleted;
		return currentObj->obj;
	}
	else {
		isDeleted = false;
		return NULL;
	}

}


FeaturePoint* FeatureList::last (bool &isDeleted) {

	currentObj = lastObj;
	if (currentObj) {
		isDeleted = currentObj->isDeleted;
		return currentObj->obj;
	}
	else {
		isDeleted = false;
		return NULL;
	}
}



FeaturePoint * FeatureList::current (bool &isDeleted) const {

	if (currentObj) {
		isDeleted = currentObj->isDeleted;
		return currentObj->obj;
	}
	isDeleted = false;
	return NULL;
}


FeaturePoint* FeatureList::next (bool &isDeleted) {

	if (currentObj->next) {
		currentObj = currentObj->next;
		isDeleted = currentObj->isDeleted;
		return currentObj->obj;
	} 
		
	isDeleted = false;
	return 0;
}


FeaturePoint* FeatureList::prev (bool &isDeleted) {

	
	if (currentObj->prev) {
		currentObj = currentObj->prev;

		return currentObj->obj;
	} 
		
	isDeleted = false;
	return 0;
}


FeaturePoint* FeatureList::getFirst (bool &isDeleted) const {

	if (firstObj) {
		isDeleted = firstObj->isDeleted;
		return firstObj->obj;
	}
	else {
		isDeleted = false;
		return NULL;
	}
}


FeaturePoint* FeatureList::getLast (bool &isDeleted) const {

	if (lastObj) {
		isDeleted = lastObj->isDeleted;
		return lastObj->obj;
	}
	else {
		isDeleted = false;
		return NULL;
	}
}

void FeatureList::setAutoDeleteEnabled(bool enable) {

	bAutoDelete = enable;
}
